local addonName, addon = ...

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local module = {name = addonName .. "-Accountant"}

function module:Initialize()
end

function module:Enable()
    -- Don't load or register anything if this plugin is disabled
    if addon:IsPluginDisabled(self.name) then
        self:CreateDummyFeed()
        return
    end

    local realmName = GetRealmName()
    module.db = addon:GetPluginDB(module.name)

    if not module.db[realmName] then
        module.db[realmName] = {}
    end

    if not module.db[realmName].chars then
        module.db[realmName].chars = {}
    end

    local playerGUID = UnitGUID("player")
    local playerName = UnitName("player")
    local _, playerClass = UnitClass("player")

    self.db = module.db[realmName]
    self.db.total = self.db.total or 0
    self.db.chars[playerGUID] = self.db.chars[playerGUID] or {name = playerName, class = playerClass}
    self.playerDB = self.db.chars[playerGUID]

    self:CreateDataFeed()
    self:RegisterEvent("PLAYER_MONEY", "MoneyUpdated")
    module:MoneyUpdated()
end

function module:MoneyUpdated()
    local previousMoney = self.playerDB.money or 0
    local currentMoney = GetMoney()

    if currentMoney then
        local moneyChanged = currentMoney - previousMoney

        if moneyChanged > 0 then
            self.playerDB.money = currentMoney
            self.db.total = self.db.total + moneyChanged
        elseif moneyChanged < 0 then
            self.playerDB.money = currentMoney
            self.db.total = self.db.total - moneyChanged
        else
            -- No change
        end
    end

    self:UpdateFeed()
end

function module:UpdateFeed()
    local newAmount = GetCoinTextureString(self.db.total)
    if newAmount == self.dataFeed.text then
        return
    end

    self.dataFeed.text = newAmount
end

function module:CreateDataFeed()
    self.dataFeed = ldb:NewDataObject(self.name, {
        type = "data source",
        text = GetCoinTextureString(0),
        OnTooltipShow = function(tooltip)
            self:OnTooltipShow(tooltip)
        end,
    })
end

local function sortByMoneyDesc(a, b)
    return module.db.chars[a].money > module.db.chars[b].money
end

function module:OnTooltipShow(tooltip)
    if not tooltip or not tooltip.AddLine then return end

    tooltip:AddLine(module.name)
    tooltip:AddLine("---")

    tooltip:AddDoubleLine("Total:", GetCoinTextureString(self.db.total))

    local sortKeys = {}
    for guid, char in pairs(self.db.chars) do
        table.insert(sortKeys, guid)
    end
    table.sort(sortKeys, sortByMoneyDesc)

    for idx, guid in ipairs(sortKeys) do
        local char = self.db.chars[guid]
        local classColor = C_ClassColor.GetClassColor(char.class)
        local title = string.format("%s%s|r:", classColor:GenerateHexColorMarkup(), char.name)
        tooltip:AddDoubleLine(title, GetCoinTextureString(char.money))
    end
end

addon:RegisterModule(module, module.name)